<?php

declare(strict_types=1);

namespace Drupal\Tests\datetime_testing\Kernel;

use Drupal\datetime_testing\TestDateTime;

/**
 * A datetime object that allows overriding the default timezone.
 *
 * Intended for testing the TestDatetime class.
 */
class TestTestDateTime extends TestDateTime {

  /**
   * The current time.
   *
   * @var string
   */
  protected string $fallbackTimezone;

  /**
   * {@inheritdoc}
   */
  public function __construct(string $time = 'now', ?string $timezone = NULL, array $settings = [], string $fallbackTimezone = 'UTC') {
    $this->fallbackTimezone = $fallbackTimezone;
    // Instantiate the parent class.
    parent::__construct($time, $timezone, $settings);
  }

  /**
   * {@inheritdoc}
   */
  protected function prepareTimezone(mixed $timezone): \DateTimeZone {
    // Override the default timezone for testing purposes.
    if (empty($timezone) && !empty($this->fallbackTimezone)) {
      return new \DateTimeZone($this->fallbackTimezone);
    }
    else {
      return parent::prepareTimezone($timezone);
    }
  }

}
